'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _immutable = require('immutable');

function doSetOp(param1, param2, op) {
  var type = param1.constructor.name;
  if (type === 'Set' || type === 'OrderedSet') {
    return param1[op](param2);
  }
  return param1.toOrderedSet()[op](param2)['to' + type]();
}

function customMerge(param1, param2, mergeFnc) {
  if (typeof mergeFnc === 'function') {
    return mergeFnc(param1, param2);
  }

  switch (mergeFnc) {
    case 'append':
      return param1.concat(param2);
    case 'prepend':
      return param2.concat(param1);
    case 'union':
      return doSetOp(param1, param2, 'union');
    default:
      throw new Error('Unknown merge function \'' + mergeFnc + '\'');
  }
}

function mergeDeep(param1, param2, schema) {
  if (!schema || (typeof schema === 'undefined' ? 'undefined' : _typeof(schema)) === 'object' && Object.keys(schema).length === 0) {
    return _immutable.Iterable.isIterable(param1) ? param1.mergeDeep(param2) : param2;
  }

  var schemaType = typeof schema === 'undefined' ? 'undefined' : _typeof(schema);
  if (schemaType === 'string' || schemaType === 'function') {
    return customMerge(param1, param2, schema);
  }

  if (schemaType !== 'object') {
    throw new Error('Invalid schema');
  }

  var immutableType = param1.constructor.name;
  var merged = new _immutable.Map();
  switch (immutableType) {
    case 'List':
      merged = new _immutable.List();break;
    case 'Map':
      merged = new _immutable.Map();break;
    case 'OrderedMap':
      merged = new _immutable.OrderedMap();break;
    case 'Set':
      merged = new _immutable.Set();break;
    case 'OrderedSet':
      merged = new _immutable.OrderedSet();break;
    case 'Stack':
      merged = new _immutable.Stack();break;
    default:
      throw new Error('Unsupported type'); // TODO add test for this
  }

  if (!_immutable.Iterable.isIterable(param1) || !_immutable.Iterable.isIterable(param2)) {
    return param2;
  }
  param1.forEach(function (value, key) {
    if (param2.has(key)) {
      var subSchema = schema ? schema[key] || schema['*'] : undefined;
      merged = merged.set(key, mergeDeep(param1.get(key), param2.get(key), subSchema));
    } else {
      merged = merged.set(key, param1.get(key));
    }
  });

  param2.forEach(function (value, key) {
    if (!param1.has(key)) {
      merged = merged.set(key, param2.get(key));
    }
  });

  return merged;
}

exports.default = mergeDeep;